#include-once
#include 'DecToBin.au3'
#include 'StringPop.au3'

; Magische Zahl zur Identifikation eines Containers
Global Const $__sMagic = 'MDC' ; Mars Data Container (Darf jeder Benutzer nach belieben anpassen, eine Referenz wre aber ganz schn)

; Die maximale Lnge der 3 Bitraten errechnet sich aus 2^2^iBitrate - 1
Global Const $__iBitrate_Data = 5 ;        | Lnge von 0 bis 4294967295 CHAR (ca. 4096 MB)
Global Const $__iBitrate_Index = 3 ;       | Lnge von 0 bis 255 CHAR
Global Const $__iBitrate_Description = 4 ; | Lnge von 0 bis 65535 CHAR (ca. 64 KB)
Global Const $__iBitlen_Elements = 12 ;    | Maximal 4096 Datenstze

; Overhead:
; MDC (3 Byte) + Bitraten (3 Byte) = 6 Byte | Einmalig
; Bits zur Verwaltung: 0 bis (2^5 + 2^3 + 2^4) Bit = 0 bis 7 Byte | Pro Datensatz bentigt

Func _Container_FromString($sFile)
	_DecToBinStartup()
	Local $iLen_Data, $iLen_Index, $iLen_Desc
	If Not StringPop($sFile, StringLen($__sMagic)) = $__sMagic Then Return
	Local $aCon = _Container_Create()
	Local $sPop = __Container_StringToBin(StringPop($sFile, 3))
	Local $iBit_Desc = _BinToDec(StringPop($sPop, -4))
	Local $iBit_Index = _BinToDec(StringPop($sPop, -3))
	Local $iBit_Data = _BinToDec(StringPop($sPop, -5))
	Local $iBit_Cnt = _BinToDec(StringPop($sPop, -12))
	Local $iSize_DataHeader = Ceiling(($iBit_Data + $iBit_Desc + $iBit_Index) / 8)
	For $i = 1 To $iBit_Cnt Step 1
		$sPop = __Container_StringToBin(StringPop($sFile, $iSize_DataHeader))
		$iLen_Desc = _BinToDec(StringPop($sPop, -$iBit_Desc))
		$iLen_Index = _BinToDec(StringPop($sPop, -$iBit_Index))
		$iLen_Data = _BinToDec(StringPop($sPop, -$iBit_Data))
		_Container_Add($aCon, StringPop($sFile, $iLen_Data), StringPop($sFile, $iLen_Index), StringPop($sFile, $iLen_Desc))
	Next
	Return $aCon
EndFunc   ;==>_Container_FromString

Func _Container_ToString(ByRef $aCon)
	_DecToBinStartup()
	Local $sFile = $__sMagic
	Local $iBit_Data = Int(Log(__Container_MaxLen($aCon, 'Data')) / Log(2)) + 1
	Local $iBit_Index = Int(Log(__Container_MaxLen($aCon, 'Index')) / Log(2)) + 1
	Local $iBit_Desc = Int(Log(__Container_MaxLen($aCon, 'Desc')) / Log(2)) + 1
	Local $sBit_Cnt = FillLeft(_DecToBin($aCon[0][0]), 12)
	Local $sBit_Data = FillLeft(_DecToBin($iBit_Data), 5)
	Local $sBit_Index = FillLeft(_DecToBin($iBit_Index), 3)
	Local $sBit_Desc = FillLeft(_DecToBin($iBit_Desc), 4)
	Local $sBit_Header = Hex(_BinToDec($sBit_Cnt & $sBit_Data & $sBit_Index & $sBit_Desc), 6)
	Local $iSize_DataHeader = Ceiling(($iBit_Data + $iBit_Desc + $iBit_Index) / 8) * 8
	$sFile &= BinaryToString('0x' & $sBit_Header)
	For $i = 1 To $aCon[0][0] Step 1
		$sBit_Header = BinaryToString('0x' & Hex(_BinToDec(FillLeft(FillLeft(_DecToBin($aCon[$i][0]), $iBit_Data) & FillLeft(_DecToBin($aCon[$i][1]), $iBit_Index) & FillLeft(_DecToBin($aCon[$i][2]), $iBit_Desc), $iSize_DataHeader)), $iSize_DataHeader / 4))
		$sFile &= $sBit_Header & $aCon[$i][3] & $aCon[$i][4] & $aCon[$i][5]
	Next
	Return $sFile
EndFunc   ;==>_Container_ToString

Func FillLeft($s, $n) ; Schwr ! Das ndern wir noch ;)
	Return StringRight(_Container_StringRepeat('0', $n) & $s, $n)
EndFunc   ;==>FillLeft

Func _Container_Create()
	Local $aCon[1][6] = [[0]]
	; [0][0] - Anzahl Datenstze
	; [0][1] - Bitrate iLen-Data
	; [0][2] - Bitrate iLen-Index
	; [0][3] - Bitrate iLen-Description
	Return $aCon
EndFunc   ;==>_Container_Create

Func _Container_StringRepeat($sString, $iRepeatCount)
	Local $sResult = ''
	While $iRepeatCount > 1
		If BitAND($iRepeatCount, 1) Then $sResult &= $sString
		$sString &= $sString
		$iRepeatCount = BitShift($iRepeatCount, 1)
	WEnd
	Return $sString & $sResult
EndFunc   ;==>_StringRepeat

Func _Container_Add(ByRef $aCon, $sData, $sIndex = '', $sDescription = '')
	If Not IsArray($aCon) Or $sData = '' Then Return
	$aCon[0][0] += 1
	ReDim $aCon[$aCon[0][0] + 1][UBound($aCon, 2)]
	__Container_ArrayAdd($aCon, $aCon[0][0], 0, 5, StringLen($sData), StringLen($sIndex), StringLen($sDescription), $sData, $sIndex, $sDescription)
EndFunc   ;==>_Container_Add

Func __Container_ArrayAdd(ByRef $aArray, $iIndex1, $iIndex2_Start, $iIndex2_End, $var1 = 0, $var2 = 0, $var3 = 0, $var4 = 0, $var5 = 0, $var6 = 0)
	Local $aData = [$var1, $var2, $var3, $var4, $var5, $var6]
	For $i = $iIndex2_Start To $iIndex2_End Step 1
		$aArray[$iIndex1][$i] = $aData[$i - $iIndex2_Start]
	Next
EndFunc

Func _Container_Get($aCon, $sIndex, $bDataOnly = False)
	If IsString($sIndex) Then
		For $i = 1 To $aCon[0][0] Step 1
			If $sIndex = $aCon[$i][4] Then
				$sIndex = $i
				ExitLoop
			EndIf
		Next
	EndIf
	Return _Container_GetByIndex($aCon, $sIndex, $bDataOnly)
EndFunc   ;==>_Container_Get

Func _Container_GetByIndex($aCon, $i, $bDataOnly)
	If $i >= UBound($aCon) Then Return
	If $bDataOnly Then Return $aCon[$i][3]
	Local $aRet = [$aCon[$i][3], $aCon[$i][4], $aCon[$i][5]]
	Return $aRet
EndFunc   ;==>_Container_GetByIndex

Func __Container_MaxLen(ByRef $aCon, $sItem)
	Local $n = $sItem = 'Data' ? 0 : $sItem = 'Index' ? 1 : 2, $iMax = 0
	For $i = 1 To $aCon[0][0] Step 1
		If $iMax < $aCon[$i][$n] Then $iMax = $aCon[$i][$n]
	Next
	Return $iMax
EndFunc   ;==>__Container_MaxLen

Func __Container_StringToBin($s) ; Geht nur fr maximal 7 Zeichen
	Return _DecToBin(Int('0x' & Hex(StringToBinary($s))))
EndFunc   ;==>StringToBin
